-- dining philosophers paradigm.

-- Solution implementing global allocation (and instrumentation)
-- The chopsticks are allocated only when both are available  
 
-- ADA solution simulating the Java monitor semantics
-- instrumented for concurrency denials ratio measurement

-- ***************     reliable, no deadlock, not fair, starvation prone

with Global; use Global;
package body Chop is

	type SticState is array(Id) of Boolean;
		
	protected Sticks is
		entry Get_LR(C : in Id; Done : out Boolean); 
		procedure Release(C : in Id);
	private 
		entry Wait(C : in Id; Done : out Boolean); 
		Available : SticState := (others => True); -- Stick availability
	    NotifyAll : Boolean := False; -- Java signal simulation
				
	end Sticks;
	
	protected body Sticks is   

 		entry Get_LR(C : in Id; Done : out Boolean) when True is
		begin
			if Available (C) and Available (C + 1) then
				Available (C) := False; Available (C + 1) := False; Done := True;
			else  
				Done := False; NotifyAll := False; requeue Wait;
			end if;						
		end Get_LR;    

 		entry Wait(C : in Id; Done : out Boolean) when NotifyAll is
		begin
			null; -- moves from the waiting queue to the ready queue 
		end Wait;    

		procedure Release(C : Id) is
		begin
			Available(C) := True; Available(C + 1) := True; 
			NotifyAll := (Wait'Count > 0); -- Java NotyfyAll simulation
		end Release;
	end Sticks;

	procedure Get_LR(C : Id) is
		OK : Boolean := False;	
	begin
		while not OK loop 
			Sticks.Get_LR(C, OK); 
		end loop;
	end Get_LR;

	procedure Release(C : Id) is begin Sticks.Release(C); end Release;

begin
	null; -- New_Line;
	-- Put_Line("*****   DINING PHILOSOPHERS WITH STICK GLOBALLY ALLOCATED ");
	-- Put_Line("*****   ADA SIMULATION OF JAVA MONITOR SOLUTION : RELIABLE, UNFEAR");
	-- Put_Line("*****   WITH CONCURRENCY RATIO INSTRUMENTATION");
end Chop;
