with Protected_Machine; 
with Ada.Numerics.Float_Random;
package body Global is

	--------------------------------------------------------------
	-- provides a random number generator with safe concurrent use
	---------------------------------------------------------------

    package Protected_Float_Random is new Protected_Machine
     ( Result			=> Float,
       Internal_State   => Ada.Numerics.Float_Random.Generator,
       Reset            => Ada.Numerics.Float_Random.Reset,
       Next_Value		=> Ada.Numerics.Float_Random.Random);
	   
	   -- the result is a Float value uniformely distributed in the range 0.0..1.0

	function Next_Hazard return Float is
	begin
		return Protected_Float_Random.Next_Protected_Value;
	end Next_Hazard;

	----------------------------------------------------------		
	-- provides mutual exclusion for text displaying on console
	-- allows to print lines of text correctly while concurrent requests are posted
	----------------------------------------------------------	
	
	package body Display_Lock is
	
	  protected Server is
		entry Acquire;
		procedure Release;
	  private
		Free : Boolean := True;
	  end Server;
	
	  protected body Server is
		entry Acquire when Free is begin Free := False; end Acquire ;
		procedure Release is begin Free := True; end Release;
	 end Server;
 	
	 procedure Acquire is begin Server.Acquire; end Acquire;
	 procedure Release is begin Server.Release; end Release;
	 
	end Display_Lock;
	----------------------------------------------------------
end Global;