-- dining philosophers paradigm.

-- The chopsticks are allocated as many as available 
-- and the allocation is completed as soon as the missing chopsticks are released.
 
-- ADA solution using the eggshell semantic of Ada protected object 

-- Looks like a Java program, however
-- once notified the waiting tasks don't leave the protected object
-- and don't releases the lock as it is done with Java concurrency semantic

-- instrumented for concurrency denials ratio measurement
-- ratio = Nb of cautious refusals versus Nb of structural refusals

-- ***************     reliable, no deadlock

with Global; use global;
with Monitoring; use Monitoring;
with Ada.Text_IO;  use Ada.Text_IO;
package body Chop is
	type SticState is array(Id) of Boolean;
	type Cardinal is new Integer range 0..2;
	type SticScore is array(Id) of Cardinal;

	protected Sticks is
		entry Get_LR(C : Id); 
		procedure Release(C : in Id);
	private 
		entry Wait(C : Id);
		Available : SticState := (others => True); -- Stick availability
		Score : SticScore := (others => 0); -- allocation state
		StillToNotify : Integer := 0;
		NotifyAll : Boolean := False; -- Java signal simulation 
	end Sticks;
	
   protected body Sticks is 
  
	    entry Get_LR(C : Id) when not NotifyAll is
		-- after each execution of Release 
		-- the wait entry queue must be fully emptied before serving Get_LR anew
	    begin
         	case Score(C) is
          		when 0 =>if Available (C) then 
							Available (C) := False;  Score(C) := 1; -- Left stick allocated 
							if Available (C + 1) then
								Available (C + 1) := False; Score(C) := 2; 
								-- Right stick also allocated in the spot
							end if;
						  end if;
		 		when 1 => if Available (C + 1) then
								Available (C + 1) := False; Score(C) := 2; 
								-- Right stick now allocated also 
							end if;
		  		when 2 => null;
        	end case;
			if Score(C) /= 2 then 
				Monitoring.InstrumentGetRefusal(C); -- counts one more refusal 
          		requeue Wait; -- Java wait simulation
			else
				Monitoring.InstrumentGetSuccessful(C); -- counts the success once
        	end if;
		end Get_LR;
	  
		entry Wait(C : Id) when NotifyAll is
		begin
			StillToNotify:= StillToNotify - 1;
			NotifyAll := (StillToNotify > 0); -- when False, opens the Get_LR barrier 
			requeue Get_LR; -- all signalled tasks remain inside the monitor
		end Wait;

		procedure Release(C : Id) is
		begin
			Available(C) := True; Available(C + 1) := True; Score(C) := 0;
			-- awakes all blocked tasks as with Java's NotifyAll
			StillToNotify:= Wait'Count ;  
			NotifyAll := (StillToNotify > 0);
			Monitoring.InstrumentRelease(C);
		end Release;
	end Sticks;
   
	procedure Get_LR(C : Id) is	
	begin
		ChopStart(C); -- starting service time for sticks allocation 
		Sticks.Get_LR(C); 
		ChopEnd(C); -- ending  service time for sticks allocation
	end Get_LR;
	procedure Release(C : Id) is begin Sticks.Release(C); end Release;	

begin
	New_Line;
	Put_Line("*****   DINING PHILOSOPHERS WITH STICK ALLOCATION AS MANY AS AVAILABLE");
	Put_Line("*****   RELIABLE ADA SOLUTION : LOOKING LIKE JAVA STYLE");
	Put_Line("*****   WITH CONCURRENCY RATIO INSTRUMENTATION");
end Chop;